/* =====================================================================
   SISTEMA : Expedio
   DATA    : 28/10/2011
   CRIAO : Andr Rodrigo

   OBSERVAES :
     - IDIDENTCAPAOPER = 03 - TRANSFERNCIA FSICA.

     - SE ESTA PROCEDURE FOR ALTERADA A PROCEDURE   "CANCELA_ATIVO_EXP"
        DEVE SER VERIFICADA.
   =====================================================================
*/

CREATE OR REPLACE PROCEDURE ATUALIZA_ATIVO_EXP(
   P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_NREXPEDICAO  IN EXPEDICAO.NREXPEDICAO%TYPE,
   P_CDOPERADOR   IN OPERADOR.CDOPERADOR%TYPE
) AS

   V_CTPARFACI         NUMBER;
   V_CTCAPA            NUMBER;
   V_NRSEQCAPAOPEBEM   VARCHAR(20);
   V_NRSEQITCAPAOPE    VARCHAR(20);
   V_NRSEQOPERABEM     VARCHAR(20);
   V_DTCAPAOPEBEM      CAPAOPERABEM.DTCAPAOPEBEM%TYPE;
   V_CDCLIENTEANTE     CLIENTE.CDCLIENTE%TYPE;
   V_CDTIPOOPERBEM     TIPOOPERBEM.CDTIPOOPERBEM%TYPE;
   V_CDMOTIVOOPER      MOTIVOOPER.CDMOTIVOOPER%TYPE;
BEGIN
   SELECT COUNT(*)
     INTO V_CTPARFACI
     FROM FCT_PARAMETROGERAL;

   IF V_CTPARFACI > 0 THEN
      V_CTCAPA := 0;
      FOR C_NOTA IN (SELECT BE.CDEMPRESA, BE.CDBEMATIVO, BE.QTATUABEMATIVO,
                            BE.CDFILIAL AS CDFILIALBEM, BE.CDCENTCUST, BE.CDSETOR,
                            BE.CDGRUPOBEM, BE.VRJADEPBEMATIVO, BE.CDTIPODEPREC,
                            BE.VRINIDEPBEM, BE.VRFALTADEPBEM, NF.CDCLIENTE,
                            FI.NRSEQORCAMENTO, NF.CDFILIAL, NF.NRLANCTONF,
                            IP.NRPEDIDO, NF.NRNOTAFISC, NF.CDSERIE,
                            FI.VRTOTALITEMORCA
                       FROM NOTAFISCAL NF, ITEMPRODUTO IT, ITEMPEDIDO IP,
                            FCT_ITEMORCAMENTO FI, BEMATIVO BE
                      WHERE NF.CDFILIAL = P_CDFILIAL
                        AND NF.NREXPEDICAO = P_NREXPEDICAO
                        AND NF.CDFILIAL = IT.CDFILIAL
                        AND NF.NRLANCTONF = IT.NRLANCTONF
                        AND IT.CDFILIALPED = IP.CDFILIAL
                        AND IT.NRPEDIDO = IP.NRPEDIDO
                        AND IT.NRITEMPED = IP.NRITEMPED
                        AND IP.NRITEMORCAMENTO = FI.NRITEMORCAMENTOPAI
                        AND FI.CDEMPRESA = BE.CDEMPRESA
                        AND FI.CDBEMATIVO = BE.CDBEMATIVO
                      ORDER BY FI.CDEMPRESA, FI.CDBEMATIVO)
      LOOP
         /*** BUSCA A LTIMA ALTERAO DO BEM  ***/
         BEGIN
            SELECT AA.DTCAPAOPEBEM, AA.CDCLIENTE
              INTO V_DTCAPAOPEBEM, V_CDCLIENTEANTE
              FROM (SELECT CA.DTCAPAOPEBEM, IT.CDCLIENTE
                      FROM CAPAOPERABEM CA, ITCAPAOPERABEM IT
                     WHERE IT.CDEMPRESA = C_NOTA.CDEMPRESA
                       AND IT.CDBEMATIVO = C_NOTA.CDBEMATIVO
                       AND CA.IDSIMULOPERABEM = 'R'
                       AND CA.CDEMPRESA = IT.CDEMPRESA
                       AND CA.IDSIMULOPERABEM = IT.IDSIMULOPERABEM
                       AND CA.NRSEQCAPAOPEBEM = IT.NRSEQCAPAOPEBEM
                     ORDER BY DTCAPAOPEBEM DESC) AA
             WHERE ROWNUM = 1;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_DTCAPAOPEBEM := NULL;
               V_CDCLIENTEANTE := NULL;
         END;

         /*** BUSCA O TIPO E O MOTIVO DA OPERAO PARAMETRIZADO PARA EXPEDIO ***/
         BEGIN
            SELECT CDTPOPERBEMEXP, CDMTOPERBEMEXP
              INTO V_CDTIPOOPERBEM, V_CDMOTIVOOPER
              FROM PARAMEMPRESA
             WHERE CDEMPRESA = C_NOTA.CDEMPRESA;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_CDTIPOOPERBEM := NULL;
               V_CDMOTIVOOPER  := NULL;
         END;

         IF V_CTCAPA = 0 THEN  /*** INSERE APENAS UMA CAPA ***/

            NOVO_CODIGO('CAPAOPERABEM' || C_NOTA.CDEMPRESA || 'R', V_NRSEQCAPAOPEBEM);
            V_NRSEQCAPAOPEBEM := SUBSTR(V_NRSEQCAPAOPEBEM, 11, 10);

            INSERT INTO CAPAOPERABEM
                (CDEMPRESA,         IDSIMULOPERABEM,    NRSEQCAPAOPEBEM,    DTCAPAOPEBEM,
                 CDTIPOOPERBEM,     CDMOTIVOOPER,       CDOPERESPOPERA,     IDIDENTCAPAOPER,
                 DTINCCAPAOPER,     CDOPEINCCAPA,       DTATUCAPAOPER,      CDOPEATUCAPA,
                 IDSITCAPAOPER,     DTCAPAOPEBEMANTE)
            VALUES
                (C_NOTA.CDEMPRESA, 'R',               V_NRSEQCAPAOPEBEM,  SYSDATE,
                 V_CDTIPOOPERBEM,  V_CDMOTIVOOPER,    P_CDOPERADOR,       '03',
                 SYSDATE,          P_CDOPERADOR,      SYSDATE,            P_CDOPERADOR,
                 '2',              V_DTCAPAOPEBEM);

            V_CTCAPA := 1;
         END IF;

         NOVO_CODIGO('OPERACAOBEM' || C_NOTA.CDEMPRESA || C_NOTA.CDBEMATIVO || 'R', V_NRSEQOPERABEM);
         V_NRSEQOPERABEM := SUBSTR(V_NRSEQOPERABEM, 11, 10);

         INSERT INTO OPERACAOBEM
            (CDEMPRESA,       CDBEMATIVO,      IDSIMULOPERACAO,   NRSEQOPERABEM,    DTOPERABEM,    CDTIPOOPERBEM,
             CDOPERESPOPERA,  QTOPEBEMNOVO,    VROPERABEM,        CDFILIAL,         CDCENTCUST,    CDSETOR,
             CDGRUPOBEM,      QTMESDEPOPENOVO, QTMESFALDEPOPEB,   QTMESJADEPOPEB,   VRTAXDEPOPENOVO,
             CDTIPODEPREC,    VRINIDEPROPEBEM, VRJADEPROPEBEM,    VRFALDEPROPEBEM,  DTINCOPERABEM,
             CDOPEINCOPERABE, DTATUOPERABEM,   CDOPEATUOPERABE ,  IDIDENTOPERABEM,  IDSITOPERABEM, CDCLIENTE,
             CDFILIALNF    ,  NRLANCTONF,      CDMOTIVOOPER,      CDSERIENFOPERA)
         VALUES
            (C_NOTA.CDEMPRESA,    C_NOTA.CDBEMATIVO,      'R',                   V_NRSEQOPERABEM,        SYSDATE,     V_CDTIPOOPERBEM,
             P_CDOPERADOR,        C_NOTA.QTATUABEMATIVO , C_NOTA.VRTOTALITEMORCA, C_NOTA.CDFILIALBEM,     C_NOTA.CDCENTCUST, C_NOTA.CDSETOR,
             C_NOTA.CDGRUPOBEM,   0,                      0,                     C_NOTA.VRJADEPBEMATIVO, 0,
             C_NOTA.CDTIPODEPREC, C_NOTA.VRINIDEPBEM,     C_NOTA.VRJADEPBEMATIVO, C_NOTA.VRFALTADEPBEM,  SYSDATE,
             P_CDOPERADOR,        SYSDATE,                P_CDOPERADOR,          '03',                   '1', C_NOTA.CDCLIENTE,
             C_NOTA.CDFILIAL,     C_NOTA.NRLANCTONF,      V_CDMOTIVOOPER,        C_NOTA.CDSERIE);

         /*** A ITCAPAOPERABEM   includa depois da OPERACAOBEM  para gravar o NRSEQOPERABEM ***/
         NOVO_CODIGO('ITCAPAOPERABEM' || C_NOTA.CDEMPRESA || 'R' || V_NRSEQCAPAOPEBEM, V_NRSEQITCAPAOPE);
         V_NRSEQITCAPAOPE := SUBSTR(V_NRSEQITCAPAOPE, 11, 10);

         INSERT INTO ITCAPAOPERABEM
             (CDEMPRESA,       IDSIMULOPERABEM, NRSEQCAPAOPEBEM, NRSEQITCAPAOPE,
              CDBEMATIVO,      QTITCAPAOPEANTE, QTITCAPAOPENOVO, VRITCAPAOPERA,
              CDFILIALITCANTE, CDFILIALITCNOVO, CDCENTCUITCANTE, CDCENTCUITCNOVO,
              CDSETORITCANTE,  CDSETORITCNOVO,  QTMESDEPITCANTE, QTMESDEPITCNOVO,
              VRTAXDEPITCANTE, VRTAXDEPITCNOVO, DTINCITCAPAOPER, DTATUITCAPAOPER,
              CDOPEATUITCAPA,  CDOPEINCITCAPA,  CDOPERESPBEMANT, CDOPERESPBEMNOV,
              CDCLIENTE     ,  CDCLIENTEANTE  , NRNOTAFIITCAPA , DSOBSITCAPAOPER,
              CDFILIALNF    ,  NRLANCTONF,      NRSEQOPERABEM)
         VALUES
             (C_NOTA.CDEMPRESA,   'R',               V_NRSEQCAPAOPEBEM,  V_NRSEQITCAPAOPE,
              C_NOTA.CDBEMATIVO,  C_NOTA.QTATUABEMATIVO, C_NOTA.QTATUABEMATIVO, C_NOTA.VRTOTALITEMORCA,
              NULL,               NULL,              NULL,               NULL,
              NULL,               NULL,              0,                  0,
              0,                  0,                 SYSDATE,            SYSDATE,
              P_CDOPERADOR,       P_CDOPERADOR,      P_CDOPERADOR,       P_CDOPERADOR,
              C_NOTA.CDCLIENTE,   V_CDCLIENTEANTE,   C_NOTA.NRNOTAFISC,
              'EXPEDIO DO PEDIDO: ' || C_NOTA.NRPEDIDO || ' - ORAMENTO: ' || C_NOTA.NRSEQORCAMENTO,
              C_NOTA.CDFILIAL,    C_NOTA.NRLANCTONF, V_NRSEQOPERABEM);
      END LOOP;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ATUALIZA_ATIVO_EXP','5.25.000'); END;
/

